#!/usr/bin/python
# -*- coding: UTF-8 -*-
# sdk for nebula

import os
import sys
import hashlib
import json
import md5
import requests
import time

def Tm():
    localtime = str(int(time.time()))
    return localtime


class Token(object):
    """docstring for Token"""

    def __init__(self, accesskey, secretkey):
        super(Token, self).__init__()
        self.accesskey = accesskey
        self.secretkey = secretkey

    # token 生成方式
    def token(self, parameter):
        m = md5.new()
        m.update(parameter)
        return m.hexdigest()


class ApiSdk(object):
    """docstring for ApiSdk"""

    def __init__(self, token, url, bucket):
        super(ApiSdk, self).__init__()
        self.token = token
        self.url = url
        self.bucket = bucket

    # 递归删除目录
    def rmDir(self, uri):
        currenttimestamp = Tm()
        parameter = self.bucket + ''.join(uri) + self.token.accesskey + self.token.secretkey + currenttimestamp
        data = json.dumps({
            "bucket": self.bucket,
            "uri": uri,
            "ak": self.token.accesskey,
            "token": self.token.token(parameter),
            "timestamp": currenttimestamp,
        })

        return requests.post(self.url+"/rmdir", data=data).text

    # 列出当前目录内容
    def fileList(self, uri, pageno, pagesize):
        currenttimestamp = Tm()
        parameter = self.bucket + uri + str(pageno) + str(pagesize) + self.token.accesskey + self.token.secretkey + currenttimestamp
        data = json.dumps({
            "bucket": self.bucket,
            "uri": uri,
            "pageno": int(pageno),
            "pagesize": int(pagesize),
            "ak": self.token.accesskey,
            "token": self.token.token(parameter),
            "timestamp": currenttimestamp,
        })

        url = self.url+"/list"
        return requests.post(url, data=data).text

    # 删除文件
    def rmFile(self, uri):
        currenttimestamp = Tm()
        parameter = self.bucket + ''.join(uri) + self.token.accesskey + self.token.secretkey + currenttimestamp
        data = json.dumps({
            "bucket": self.bucket,
            "uri": uri,
            "ak": self.token.accesskey,
            "token": self.token.token(parameter),
            "timestamp": currenttimestamp,
        })
        
        return requests.post(self.url+"/rm", data=data).text

    def createMultipartPostUrl(self, exipiretime, filepath, uploadUri, md5FileStr):
        ft = os.stat(filepath)
        filesize = str(ft.st_size)
        currenttimestamp = Tm()
        parameter = self.token.accesskey + self.token.secretkey + currenttimestamp + self.bucket + filesize + uploadUri + exipiretime + md5FileStr
        url = self.url+"/upload"
        return url + "?" + "bucket=" + self.bucket + \
            "&filesize=" + filesize + \
            "&uri=" + uploadUri + \
            "&expiretime=" + exipiretime + \
            "&sign=" + self.token.token(parameter) + \
	    "&mcheck=" + md5FileStr + \
	    "&ct=" + currenttimestamp + \
	    "&ak=" + self.token.accesskey

    # 上传文件
    def uploadFile(self, exipiretime, filePath, uploadUri):
        
        f = open(filePath, 'rb')
        files = {'file': open(filePath, "rb")}
   
        # 计算文件大小
        ft = os.stat(filePath)
        filesize = ft.st_size
        
        m = 2 << 19
        myhash = hashlib.md5()
        if filesize / m >= 100:
            block_size = 8192  #8KB
            while True:
                data = f.read(block_size)
                if not data:
                    break
                myhash.update(data)
            md5FileStr = myhash.hexdigest()
        else:
            myhash.update(f.read())
            md5FileStr = myhash.hexdigest()

        url = self.createMultipartPostUrl(
            exipiretime,
            filePath,
            uploadUri,
            md5FileStr)

	# 添加证书认证
	s = requests.Session()
	res = s.post(url, verify=True, files=files).text	

	f.close()
	return res
